import { useEffect, useState } from 'react'
import { Link, useParams, useNavigate } from 'react-router-dom'
import ReactMarkdown from 'react-markdown'
import axios from 'axios'
import { API_BASE_URL } from '@/config'


const WikiDetailPage = () => {
  const { name } = useParams ()

  const navigate = useNavigate ()

  const [markdown, setMarkdown] = useState<string | null> (null)

  useEffect (() => {
    if (/^\d+$/.test (name))
      {
        void (axios.get (`${ API_BASE_URL }/wiki/${ name }`)
              .then (res => navigate (`/wiki/${ res.data.title }`, { replace: true })))
        return
      }

    void (axios.get (`${ API_BASE_URL }/wiki/title/${ encodeURIComponent (name) }`)
          .then (res => setMarkdown (res.data))
          .catch (() => setMarkdown (null)))
  }, [name])

  return (
      <div className="prose mx-auto p-4">
        <ReactMarkdown components={{ a: (
            ({ href, children }) => (href?.startsWith ('/')
                                     ? <Link to={href!}>{children}</Link>
                                     : <a href={href} target="_blank" rel="noopener noreferrer">{children}</a>)) }}>
          {markdown || `このページは存在しません。[新規作成してください](/wiki/new?title=${ name })。`}
        </ReactMarkdown>
      </div>)
}


export default WikiDetailPage
